/*------------------------------------------------------------------------------
Inference on Winners 
Authors: Isaiah Andrews, Toru Kitagawa, Adam McCloskey
Code Author: Noah Siderhurst (noahside@mit.edu)
Version 1 - February 2024
------------------------------------------------------------------------------*/
clear all
macro drop _all

//Path Macros
global home_dir "E:\Inference on Winners"
cd "$home_dir"
*pause on

//User Inputs
local ndraws = 10000
	*note: this is near the maximum number of draws for Stata SE
local alpha = 0.05
local beta = 0.005
local mu_format "x y"
local K = 13
local mu = "mu.csv"
local sigma = "sigma_xy.csv"

/*------------------------------------------------------------------------------
Step 1: Preliminaries
------------------------------------------------------------------------------*/

//Saving Mu and Sigma inputs as matrices
import delimited `mu', clear
mkmat v1, mat(mu)
mkmat v1 in 1/`K', mat(x)
local y_start = `K' + 1
local y_end = 2 * `K'
cap mkmat v1 in `y_start'/`y_end', mat(y)

import delimited `sigma', clear
describe, varlist
mkmat `r(varlist)', mat(sigma)

** Replicate the vector of estimates if no y 
if "`mu_format'" == "x" {
	mat y = x 
}

svmat x 
summ x1 
local THETA_TILDE = r(max)

svmat y 
summ y1 if x == r(max)
local YTILDE = r(max)
di `YTILDE'

svmat mu

gen ytilde = (y == `YTILDE')	
mkmat ytilde, mat(THETA_TILDE)
mat li THETA_TILDE

gen tid = _n 
gsort -ytilde
local THETA_TILDE_tid = tid

** Draw from standard normal distribution with fixed seed 
set seed 100
matrix FEED_IN = J(`ndraws', `K', 0) 
forval i = 1/`ndraws' {
	forval j = 1/`K' {
		matrix FEED_IN[`i', `j'] = rnormal()
	}
}

** Replicate the variance matrix if no y
if "`mu_format'" == "x" {
	matrix define ones = (1, 1 \ 1, 1)
	mat sigma = ones#sigma
	mat sigma = ones#sigma
}

** Variance of all the estimates
mat SIGMAY = sigma[`K'+1 .. 2*`K', `K'+1 .. 2*`K']
mat li SIGMAY

** Variance of the winning arm
local SIGMAYTILDE = sigma[`K' + `THETA_TILDE_tid', `K' + `THETA_TILDE_tid']
di `SIGMAYTILDE'

** Covariance of the winning arm and other arms
matrix SIGMAXYTILDE_VEC = sigma[`K' + `THETA_TILDE_tid', 1..`K']
matrix SIGMAXYTILDE_VEC = SIGMAXYTILDE_VEC' 
matrix list SIGMAXYTILDE_VEC

** Covariance of the winning arm
local SIGMAXYTILDE = sigma[`THETA_TILDE_tid', `K' + `THETA_TILDE_tid']
di `SIGMAXYTILDE'

//Normalised difference between each treatment arm and the winning arm
sort tid
svmat SIGMAXYTILDE_VEC
gen ZTILDE = x1 - (SIGMAXYTILDE_VEC1 / `SIGMAYTILDE') * `YTILDE'

//Truncation values
gen LTILDE = .
gen UTILDE = .
gen SIGMAXYTILDE = `SIGMAXYTILDE'
mat SE = vecdiag(SIGMAY)'
svmat SE

** Lower
gen IND_L = (SIGMAXYTILDE > SIGMAXYTILDE_VEC1)
replace LTILDE = -20 * SE1 if IND_L==0 // 20 * SE approximates infinity
replace LTILDE = (`SIGMAYTILDE' * (ZTILDE - ZTILDE[`THETA_TILDE_tid'])) / (SIGMAXYTILDE - SIGMAXYTILDE_VEC1) if IND_L==1

summ LTILDE
local LTILDE = r(max)

** Upper
gen IND_U = (SIGMAXYTILDE < SIGMAXYTILDE_VEC1)
replace UTILDE = 20 * SE1 if IND_U==0 // 20 * SE approximates infinity
replace UTILDE = (`SIGMAYTILDE' * (ZTILDE - ZTILDE[`THETA_TILDE_tid'])) / (SIGMAXYTILDE - SIGMAXYTILDE_VEC1) if IND_U==1

summ UTILDE
local UTILDE = r(min)

di `LTILDE'
di `UTILDE'

** Hybrid Level
gen VTILDE = .
gen IND_V = (SIGMAXYTILDE_VEC1 == SIGMAXYTILDE)

replace VTILDE = 0 if IND_V==0
replace VTILDE = -1 * (ZTILDE - ZTILDE[`THETA_TILDE_tid']) if IND_V==1 

summ VTILDE
local VTILDE = r(min)
di `VTILDE'

local GAMMA = (`alpha' - `beta') / (1 - `beta')

//Compute the standard deviations of the estimates
mata:
	SIGMAY = st_matrix("SIGMAY")

	function check_diag(real matrix D) 
	{
		C = isdiagonal(D)
		x = C
		st_numscalar("diag", x)
		st_local("diag", strofreal(x))
	}
	
	check_diag(SIGMAY)
end

di `diag'

if `diag' == 1 {
	mat MFACTORS = SIGMAY 
	forval i = 1/`= rowsof(SIGMAY)' { 
		forval j = 1/`= colsof(SIGMAY)' { 
			mat MFACTORS[`i', `j'] = sqrt(SIGMAY[`i', `j']) 
		}
	} 
}

else if `diag' == 0 {
	matrix symeigen X v = SIGMAY
	
	mata: 
		vals = st_matrix("v")
		vecs = st_matrix("X")

		MFACTORS = vecs * diag(sqrt(vals)) * invsym(vecs)
		
		st_matrix("MFACTORS", MFACTORS)

	end
}

//Rescale the standard normal draws using SD and compute the critical values
mata:
	FEED_IN = st_matrix("FEED_IN")
	MFACTORS = st_matrix("MFACTORS")
	SIGMAY = st_matrix("SIGMAY")
	
	GAUSSIAN_MAX = colmax((abs(FEED_IN * MFACTORS))' :/ sqrt(diagonal(SIGMAY)))
	GAUSSIAN_MAX = GAUSSIAN_MAX'
	
	st_matrix("GAUSSIAN_MAX", GAUSSIAN_MAX)

end

svmat GAUSSIAN_MAX
local unif_beta = (1-`beta')*100
local unif = (1-`alpha')*100

_pctile GAUSSIAN_MAX1, p(`unif', `unif_beta')

local CV_UNIF_BETA = r(r2)
local CV_UNIF = r(r1)

di `CV_UNIF_BETA'
di `CV_UNIF'

mata:
	TOL = sqrt(epsilon(1))
	st_numscalar("TOL", TOL)
	st_local("TOL", strofreal(TOL))
end 
di `TOL'

/*------------------------------------------------------------------------------
Step 2: Median Unbiased Estimate
------------------------------------------------------------------------------*/

//Expand grid until function crosses zero
local SCALE `K'
qui{
	forvalues iter=1/1000 {
		if missing(`"`MUGRIDS'"') {
			local MUGRIDS "0 0 0"
		}
		local PREVIOUS_LINE `MUGRIDS'
		*noisily di "previous line is `PREVIOUS_LINE'"
		local SIGMA = sqrt(`SIGMAYTILDE')
		if missing(`"`MUGRIDSL'"') {
			local MUGRIDSL = `YTILDE' - (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSL'
		if missing(`"`MUGRIDSU'"') {
			local MUGRIDSU = `YTILDE' + (`SCALE' * `SIGMA')
		}
		noisily di `MUGRIDSU'
		local SIZE 0.5
		local MUGRIDS = "`MUGRIDSL' `MUGRIDSU'"
		noisily di "new mugrid is `MUGRIDS'"
	
	local num 1
	clear
	set obs 2

	foreach MU in `MUGRIDS' {
		local lower = (`LTILDE'-`MU')/`SIGMA'
		local upper = (`UTILDE'-`MU')/`SIGMA'
		local cdf_val_`num' = (`YTILDE'-`MU')/`SIGMA'

		cap gen ptrn = .
		replace ptrn = (normal(`cdf_val_`num'') - normal(`lower')) / (normal(`upper') - normal(`lower')) in `num'
		 
		cap gen tail = . 
		replace tail = ptrn[`num'] - (1-`SIZE') in `num'
		
		*noisily di "rep number `num'"
		local num = `num'+1
		
	} 

	gen sign = . 
	replace sign = -1 if tail < 0
	replace sign = 0 if tail == 0
	replace sign = 1 if tail > 0
	
	egen max_sign = max(sign)
	egen min_sign = min(sign)
	gen abs_diff = abs(max_sign-min_sign)
	
	* halt condition 
	if abs_diff > `TOL' {
		noisily di "function crosses zero after `iter' expansions"
		continue, break
	}
	else {
		local SCALE = 2 * `K'
	}
}
} //quietly	
macro drop _MUGRIDS _lower _upper _cdf_val* _tail* _PREVIOUS_LINE _MUGRIDSL ///
_MUGRIDSU _MUGRIDSM _num

//Bisection search 
qui{
	forvalues iter=1/1000 {
		if missing(`"`MUGRIDS'"') {
			local MUGRIDS "0 0 0"
		}
		local PREVIOUS_LINE `MUGRIDS'
		*noisily di "previous line is `PREVIOUS_LINE'"
		local SIGMA = sqrt(`SIGMAYTILDE')
		*local SCALE `K' 
		if missing(`"`MUGRIDSL'"') {
			local MUGRIDSL = `YTILDE' - (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSL'
		if missing(`"`MUGRIDSU'"') {
			local MUGRIDSU = `YTILDE' + (`SCALE' * `SIGMA')
		}
		noisily di `MUGRIDSU'
		local MUGRIDSM = (`MUGRIDSL' + `MUGRIDSU')/2
		local SIZE 0.5
		local MUGRIDS = "`MUGRIDSL' `MUGRIDSM' `MUGRIDSU'"
		noisily di "new mugrid is `MUGRIDS'"
	
	local num 1
	clear
	set obs 3

	foreach MU in `MUGRIDS' {
		local lower = (`LTILDE'-`MU')/`SIGMA'
		local upper = (`UTILDE'-`MU')/`SIGMA'
		local cdf_val_`num' = (`YTILDE'-`MU')/`SIGMA'

		cap gen ptrn = .
		replace ptrn = (normal(`cdf_val_`num'') - normal(`lower')) / (normal(`upper') - normal(`lower')) in `num'
		 
		cap gen tail = . 
		replace tail = ptrn[`num'] - (1-`SIZE') in `num'
		
		*noisily di "rep number `num'"
		local num = `num'+1
		
	} 

	gen sign = . 
	replace sign = -1 if tail < 0
	replace sign = 0 if tail == 0
	replace sign = 1 if tail > 0
	
	tokenize `MUGRIDS'
	gen mugrids = .
	replace mugrids = `1' in 1 
	replace mugrids = `2' in 2
	replace mugrids = `3' in 3
	
	tokenize `PREVIOUS_LINE'
	gen prev_line = .
	replace prev_line = `1' in 1
	replace prev_line = `2' in 2
	replace prev_line = `3' in 3
	
	gen diff = abs(mugrids - prev_line)
	egen diff_max = max(diff)
	
	* halt conditions 
	if diff_max == 0 {
		noisily di "stopping"
		continue, break
	}
	
	gen abs_int_2 = abs(tail[2])
	if abs_int_2 < `TOL' {
		noisily di "stopping"
		continue, break
	}
	
	gen abs_mugrids_u_l_diff = abs(mugrids[3]-mugrids[1])
	if abs_mugrids_u_l_diff < `TOL'{
		noisily di "stopping"
		continue, break
	} 
	
	if sign[1] == sign[2] {
		local MUGRIDSL `MUGRIDSM'
		*noisily di `MUGRIDSL'
		noisily di "L updated, continuing.."
		continue 
	}
	
	if sign[3] == sign[2] {
		local MUGRIDSU `MUGRIDSM'
		*noisily di `MUGRIDSU'
		noisily di "U updated, continuing..."
		continue 
	}

}
	
} //quietly	

global median_unbiased_estimate = mugrids[2]
macro drop _MUGRIDS _lower _upper _cdf_val* _tail* _PREVIOUS_LINE _MUGRIDSL ///
_MUGRIDSU _MUGRIDSM _num

/*------------------------------------------------------------------------------
Step 3: Equal-Tailed CI Upper
------------------------------------------------------------------------------*/

clear
local SCALE `K'
qui{
	forvalues iter=1/1000 {
		if missing(`"`MUGRIDS'"') {
			local MUGRIDS "0 0 0"
		}
		local PREVIOUS_LINE `MUGRIDS'
		*noisily di "previous line is `PREVIOUS_LINE'"
		local SIGMA = sqrt(`SIGMAYTILDE')
		local SCALE `K' 
		if missing(`"`MUGRIDSL'"') {
			local MUGRIDSL = `YTILDE' - (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSL'
		if missing(`"`MUGRIDSU'"') {
			local MUGRIDSU = `YTILDE' + (`SCALE' * `SIGMA')
		}
		noisily di `MUGRIDSU'
		local SIZE = 1-`alpha'/2
		local MUGRIDS = "`MUGRIDSL' `MUGRIDSU'"
		noisily di "new mugrid is `MUGRIDS'"
	
	local num 1
	clear
	set obs 2

	foreach MU in `MUGRIDS' {
		local lower = (`LTILDE'-`MU')/`SIGMA'
		local upper = (`UTILDE'-`MU')/`SIGMA'
		local cdf_val_`num' = (`YTILDE'-`MU')/`SIGMA'

		cap gen ptrn = .
		replace ptrn = (normal(`cdf_val_`num'') - normal(`lower')) / (normal(`upper') - normal(`lower')) in `num'
		 
		cap gen tail = . 
		replace tail = ptrn[`num'] - (1-`SIZE') in `num'
		
		*noisily di "rep number `num'"
		local num = `num'+1
		
	} 

	gen sign = . 
	replace sign = -1 if tail < 0
	replace sign = 0 if tail == 0
	replace sign = 1 if tail > 0
	
	egen max_sign = max(sign)
	egen min_sign = min(sign)
	gen abs_diff = abs(max_sign-min_sign)
	
	* halt condition 
	if abs_diff > `TOL' {
		noisily di "function crosses zero after `iter' expansions"
		continue, break
	}
	else {
		local SCALE = 2 * `K'
	}
}
} //quietly	
macro drop _MUGRIDS _lower _upper _cdf_val* _tail* _PREVIOUS_LINE _MUGRIDSL ///
_MUGRIDSU _MUGRIDSM _num

qui{
	forvalues iter=1/1000 {
		if missing(`"`MUGRIDS'"') {
			local MUGRIDS "0 0 0"
		}
		local PREVIOUS_LINE `MUGRIDS'
		*noisily di "previous line is `PREVIOUS_LINE'"
		local SIGMA = sqrt(`SIGMAYTILDE')
		local SCALE `K' 
		if missing(`"`MUGRIDSL'"') {
			local MUGRIDSL = `YTILDE' - (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSL'
		if missing(`"`MUGRIDSU'"') {
			local MUGRIDSU = `YTILDE' + (`SCALE' * `SIGMA')
		}
		noisily di `MUGRIDSU'
		local MUGRIDSM = (`MUGRIDSL' + `MUGRIDSU')/2
		local SIZE = 1-`alpha'/2
		local MUGRIDS = "`MUGRIDSL' `MUGRIDSM' `MUGRIDSU'"
		noisily di "new mugrid is `MUGRIDS'"
	
	local num 1
	clear
	set obs 3

	foreach MU in `MUGRIDS' {
		local lower = (`LTILDE'-`MU')/`SIGMA'
		local upper = (`UTILDE'-`MU')/`SIGMA'
		local cdf_val_`num' = (`YTILDE'-`MU')/`SIGMA'

		cap gen ptrn = .
		replace ptrn = (normal(`cdf_val_`num'') - normal(`lower')) / (normal(`upper') - normal(`lower')) in `num'
		 
		cap gen tail = . 
		replace tail = ptrn[`num'] - (1-`SIZE') in `num'
		
		*noisily di "rep number `num'"
		local num = `num'+1
		
	} 

	gen sign = . 
	replace sign = -1 if tail < 0
	replace sign = 0 if tail == 0
	replace sign = 1 if tail > 0
	
	tokenize `MUGRIDS'
	gen mugrids = .
	replace mugrids = `1' in 1 
	replace mugrids = `2' in 2
	replace mugrids = `3' in 3
	
	tokenize `PREVIOUS_LINE'
	gen prev_line = .
	replace prev_line = `1' in 1
	replace prev_line = `2' in 2
	replace prev_line = `3' in 3
	
	gen diff = abs(mugrids - prev_line)
	egen diff_max = max(diff)
	
	* halt conditions 
	if diff_max == 0 {
		noisily di "stopping"
		continue, break
	}
	
	gen abs_int_2 = abs(tail[2])
	if abs_int_2 < `TOL' {
		noisily di "stopping"
		continue, break
	}
	
	gen abs_mugrids_u_l_diff = abs(mugrids[3]-mugrids[1])
	if abs_mugrids_u_l_diff < `TOL'{
		noisily di "stopping"
		continue, break
	} 
	
	if sign[1] == sign[2] {
		local MUGRIDSL `MUGRIDSM'
		*noisily di `MUGRIDSL'
		noisily di "L updated, continuing.."
		continue 
	}
	
	if sign[3] == sign[2] {
		local MUGRIDSU `MUGRIDSM'
		*noisily di `MUGRIDSU'
		noisily di "U updated, continuing..."
		continue 
	}

}
	
} //quietly	

global equi_ci_upper = mugrids[2]
macro drop _MUGRIDS _lower _upper _cdf_val* _tail* _PREVIOUS_LINE _MUGRIDSL ///
_MUGRIDSU _MUGRIDSM _num

/*------------------------------------------------------------------------------
Step 4: Equal-Tailed CI Lower
------------------------------------------------------------------------------*/

clear
local SCALE `K'
qui{
	forvalues iter=1/1000 {
		if missing(`"`MUGRIDS'"') {
			local MUGRIDS "0 0 0"
		}
		local PREVIOUS_LINE `MUGRIDS'
		*noisily di "previous line is `PREVIOUS_LINE'"
		local SIGMA = sqrt(`SIGMAYTILDE')
		local SCALE `K' 
		if missing(`"`MUGRIDSL'"') {
			local MUGRIDSL = `YTILDE' - (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSL'
		if missing(`"`MUGRIDSU'"') {
			local MUGRIDSU = `YTILDE' + (`SCALE' * `SIGMA')
		}
		noisily di `MUGRIDSU'
		local SIZE = `alpha'/2
		local MUGRIDS = "`MUGRIDSL' `MUGRIDSU'"
		noisily di "new mugrid is `MUGRIDS'"
	
	local num 1
	clear
	set obs 2

	foreach MU in `MUGRIDS' {
		local lower = (`LTILDE'-`MU')/`SIGMA'
		local upper = (`UTILDE'-`MU')/`SIGMA'
		local cdf_val_`num' = (`YTILDE'-`MU')/`SIGMA'

		cap gen ptrn = .
		replace ptrn = (normal(`cdf_val_`num'') - normal(`lower')) / (normal(`upper') - normal(`lower')) in `num'
		 
		cap gen tail = . 
		replace tail = ptrn[`num'] - (1-`SIZE') in `num'
		
		*noisily di "rep number `num'"
		local num = `num'+1
		
	} 

	gen sign = . 
	replace sign = -1 if tail < 0
	replace sign = 0 if tail == 0
	replace sign = 1 if tail > 0
	
	egen max_sign = max(sign)
	egen min_sign = min(sign)
	gen abs_diff = abs(max_sign-min_sign)
	
	* halt condition 
	if abs_diff > `TOL' {
		noisily di "function crosses zero after `iter' expansions"
		continue, break
	}
	else {
		local SCALE = 2 * `K'
	}
}
} //quietly	
macro drop _MUGRIDS _lower _upper _cdf_val* _tail* _PREVIOUS_LINE _MUGRIDSL ///
_MUGRIDSU _MUGRIDSM _num

qui{
	forvalues iter=1/1000 {
		if missing(`"`MUGRIDS'"') {
			local MUGRIDS "0 0 0"
		}
		local PREVIOUS_LINE `MUGRIDS'
		*noisily di "previous line is `PREVIOUS_LINE'"
		local SIGMA = sqrt(`SIGMAYTILDE')
		local SCALE `K' 
		if missing(`"`MUGRIDSL'"') {
			local MUGRIDSL = `YTILDE' - (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSL'
		if missing(`"`MUGRIDSU'"') {
			local MUGRIDSU = `YTILDE' + (`SCALE' * `SIGMA')
		}
		noisily di `MUGRIDSU'
		local MUGRIDSM = (`MUGRIDSL' + `MUGRIDSU')/2
		local SIZE = `alpha'/2
		local MUGRIDS = "`MUGRIDSL' `MUGRIDSM' `MUGRIDSU'"
		noisily di "new mugrid is `MUGRIDS'"
	
	local num 1
	clear
	set obs 3

	foreach MU in `MUGRIDS' {
		local lower = (`LTILDE'-`MU')/`SIGMA'
		local upper = (`UTILDE'-`MU')/`SIGMA'
		local cdf_val_`num' = (`YTILDE'-`MU')/`SIGMA'

		cap gen ptrn = .
		replace ptrn = (normal(`cdf_val_`num'') - normal(`lower')) / (normal(`upper') - normal(`lower')) in `num'
		 
		cap gen tail = . 
		replace tail = ptrn[`num'] - (1-`SIZE') in `num'
		
		*noisily di "rep number `num'"
		local num = `num'+1
		
	} 

	gen sign = . 
	replace sign = -1 if tail < 0
	replace sign = 0 if tail == 0
	replace sign = 1 if tail > 0
	
	tokenize `MUGRIDS'
	gen mugrids = .
	replace mugrids = `1' in 1 
	replace mugrids = `2' in 2
	replace mugrids = `3' in 3
	
	tokenize `PREVIOUS_LINE'
	gen prev_line = .
	replace prev_line = `1' in 1
	replace prev_line = `2' in 2
	replace prev_line = `3' in 3
	
	gen diff = abs(mugrids - prev_line)
	egen diff_max = max(diff)
	
	* halt conditions 
	if diff_max == 0 {
		noisily di "stopping"
		continue, break
	}
	
	gen abs_int_2 = abs(tail[2])
	if abs_int_2 < `TOL' {
		noisily di "stopping"
		continue, break
	}
	
	gen abs_mugrids_u_l_diff = abs(mugrids[3]-mugrids[1])
	if abs_mugrids_u_l_diff < `TOL'{
		noisily di "stopping"
		continue, break
	} 
	
	if sign[1] == sign[2] {
		local MUGRIDSL `MUGRIDSM'
		*noisily di `MUGRIDSL'
		noisily di "L updated, continuing.."
		continue 
	}
	
	if sign[3] == sign[2] {
		local MUGRIDSU `MUGRIDSM'
		*noisily di `MUGRIDSU'
		noisily di "U updated, continuing..."
		continue 
	}

}
	
} //quietly	

global equi_ci_lower = mugrids[2]
macro drop _MUGRIDS _lower _upper _cdf_val* _tail* _PREVIOUS_LINE _MUGRIDSL ///
_MUGRIDSU _MUGRIDSM _num

/*------------------------------------------------------------------------------
Step 5: Hybrid Median Estimate 
------------------------------------------------------------------------------*/

clear
local SCALE `K'
qui{
	forvalues iter=1/1000 {
		if missing(`"`MUGRIDS'"') {
			local MUGRIDS "0 0"
		}
		local PREVIOUS_LINE `MUGRIDS'
		*noisily di "previous line is `PREVIOUS_LINE'"
		local SIGMA = sqrt(`SIGMAYTILDE')
		local SCALE `K' 
		if missing(`"`MUGRIDSL'"') {
			local MUGRIDSL = `YTILDE' - (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSL'
		if missing(`"`MUGRIDSU'"') {
			local MUGRIDSU = `YTILDE' + (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSU'
		local SIZE = .5
		local MUGRIDS = "`MUGRIDSL' `MUGRIDSU'"
		noisily di "new mugrid is `MUGRIDS'"
  
		local MUGRIDSU_plus = `MUGRIDSU' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSL_plus = `MUGRIDSL' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSU_minus = `MUGRIDSU' - `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSL_minus = `MUGRIDSL' - `CV_UNIF_BETA' * `SIGMA'
			
		clear
		set obs 2
			
		gen mugrids_plus = .
		replace mugrids_plus = `MUGRIDSL_plus' in 1
		replace mugrids_plus = `MUGRIDSU_plus' in 2
		
		gen mugrids_minus = .
		replace mugrids_minus = `MUGRIDSL_minus' in 1
		replace mugrids_minus = `MUGRIDSU_minus' in 2
		
		gen L = `LTILDE'
		gen U = `UTILDE'
		
		egen lbound = rowmax(L mugrids_minus)
		egen ubound = rowmin(U mugrids_plus)

		if lbound[1] < ubound[1] {
			**ptrn
			local lower = (lbound[1]-`MUGRIDSL')/`SIGMA'
			local upper = (ubound[1]-`MUGRIDSL')/`SIGMA'
			local cdf_val_1 = (`YTILDE'-`MUGRIDSL')/`SIGMA'

			cap gen ptrn = .
			replace ptrn = (normal(`cdf_val_1') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 1
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[1] - (1-`SIZE') in 1
		}
		
		if lbound[2] < ubound[2] {
			**ptrn
			local lower = (lbound[2]-`MUGRIDSU')/`SIGMA'
			local upper = (ubound[2]-`MUGRIDSU')/`SIGMA'
			local cdf_val_2 = (`YTILDE'-`MUGRIDSU')/`SIGMA'

			cap gen ptrn = . 
			replace ptrn = (normal(`cdf_val_2') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 2
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[2] - (1-`SIZE') in 2
		}
		
		*cap gen tail = .
		replace tail = 2 if missing(tail)
		
		gen sign = . 
		replace sign = -1 if tail < 0
		replace sign = 0 if tail == 0
		replace sign = 1 if tail > 0
		
		egen max_sign = max(sign)
		egen min_sign = min(sign)
		gen abs_diff = abs(max_sign-min_sign)
		
		* halt condition 
		if abs_diff > `TOL' {
			noisily di "function crosses zero after `iter' expansions"
			continue, break
		}
		else {
			local SCALE = 2 * `K'
			continue 
		}	
	}
}
macro drop _MUGRIDS _lower* _upper* _cdf_val* _tail* _PREVIOUS_LINE _MUGRIDSL ///
_MUGRIDSU _MUGRIDSM _num

clear
qui{
	forvalues iter=1/1000 {
		if missing(`"`MUGRIDS'"') {
			local MUGRIDS "0 0 0"
		}
		local PREVIOUS_LINE `MUGRIDS'
		*noisily di "previous line is `PREVIOUS_LINE'"
		local SIGMA = sqrt(`SIGMAYTILDE')
		local SCALE `K' 
		if missing(`"`MUGRIDSL'"') {
			local MUGRIDSL = `YTILDE' - (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSL'
		if missing(`"`MUGRIDSU'"') {
			local MUGRIDSU = `YTILDE' + (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSU'
		local MUGRIDSM = (`MUGRIDSL' + `MUGRIDSU')/2
		local SIZE = .5
		local MUGRIDS = "`MUGRIDSL' `MUGRIDSM' `MUGRIDSU'"
		noisily di "new mugrid is `MUGRIDS'"
  
		local MUGRIDSU_plus = `MUGRIDSU' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSM_plus = `MUGRIDSM' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSL_plus = `MUGRIDSL' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSU_minus = `MUGRIDSU' - `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSM_minus = `MUGRIDSM' - `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSL_minus = `MUGRIDSL' - `CV_UNIF_BETA' * `SIGMA'
			
		clear
		set obs 3
			
		gen mugrids_plus = .
		replace mugrids_plus = `MUGRIDSL_plus' in 1
		replace mugrids_plus = `MUGRIDSM_plus' in 2
		replace mugrids_plus = `MUGRIDSU_plus' in 3
		
		gen mugrids_minus = .
		replace mugrids_minus = `MUGRIDSL_minus' in 1
		replace mugrids_minus = `MUGRIDSM_minus' in 2
		replace mugrids_minus = `MUGRIDSU_minus' in 3
		
		gen L = `LTILDE'
		gen U = `UTILDE'
		
		egen lbound = rowmax(L mugrids_minus)
		egen ubound = rowmin(U mugrids_plus)

		if lbound[1] < ubound[1] {
			**ptrn
			local lower = (lbound[1]-`MUGRIDSL')/`SIGMA'
			local upper = (ubound[1]-`MUGRIDSL')/`SIGMA'
			local cdf_val_1 = (`YTILDE'-`MUGRIDSL')/`SIGMA'

			cap gen ptrn = .
			replace ptrn = (normal(`cdf_val_1') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 1
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[1] - (1-`SIZE') in 1
		}
		
		if lbound[2] < ubound[2] {
			**ptrn
			local lower = (lbound[2]-`MUGRIDSM')/`SIGMA'
			local upper = (ubound[2]-`MUGRIDSM')/`SIGMA'
			local cdf_val_2 = (`YTILDE'-`MUGRIDSM')/`SIGMA'

			cap gen ptrn = . 
			replace ptrn = (normal(`cdf_val_2') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 2
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[2] - (1-`SIZE') in 2
		}
		
		if lbound[3] < ubound[3] {
			**ptrn
			local lower = (lbound[3]-`MUGRIDSU')/`SIGMA'
			local upper = (ubound[3]-`MUGRIDSU')/`SIGMA'
			local cdf_val_3 = (`YTILDE'-`MUGRIDSU')/`SIGMA'

			cap gen ptrn = . 
			replace ptrn = (normal(`cdf_val_3') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 3
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[3] - (1-`SIZE') in 3
		}
		
	tokenize `PREVIOUS_LINE'
	gen prev_line = .
	replace prev_line = `1' in 1
	replace prev_line = `2' in 2
	replace prev_line = `3' in 3
	
	tokenize `MUGRIDS'
	gen mugrids = .
	replace mugrids = `1' in 1 
	replace mugrids = `2' in 2
	replace mugrids = `3' in 3
	
	replace tail = 2 if missing(tail)
	gen sign = . 
	replace sign = -1 if tail < 0
	replace sign = 0 if tail == 0
	replace sign = 1 if tail > 0
	
	gen diff = abs(mugrids[3] - mugrids[1])
	
	* halt conditions 
	if diff < `TOL' {
		noisily di "stopping"
		continue, break
	}
	
	gen abs_int_2 = abs(tail[2])
	if abs_int_2 < `TOL' {
		noisily di "stopping"
		continue, break
	}
	
	gen abs_mugrids_u_l_diff = abs(mugrids[3]-mugrids[1])
	if abs_mugrids_u_l_diff < `TOL'{
		noisily di "stopping"
		continue, break
	} 
	
	if sign[1] == sign[2] {
		local MUGRIDSL `MUGRIDSM'
		*noisily di `MUGRIDSL'
		noisily di "L updated, continuing.."
		continue 
	}
	
	if sign[3] == sign[2] {
		local MUGRIDSU `MUGRIDSM'
		*noisily di `MUGRIDSU'
		noisily di "U updated, continuing..."
		continue 
	}
	
	}
}
macro drop _MUGRIDS _lower* _upper* _cdf_val* _tail* _PREVIOUS_LINE _MUGRIDSL ///
_MUGRIDSU _MUGRIDSM _num
global hyb_med_est = mugrids[2]

/*------------------------------------------------------------------------------
Step 6: Hybrid Equal-Tailed CI Upper
------------------------------------------------------------------------------*/

clear
local SCALE `K'
qui{
	forvalues iter=1/1000 {
		if missing(`"`MUGRIDS'"') {
			local MUGRIDS "0 0"
		}
		local PREVIOUS_LINE `MUGRIDS'
		*noisily di "previous line is `PREVIOUS_LINE'"
		local SIGMA = sqrt(`SIGMAYTILDE')
		local SCALE `K' 
		if missing(`"`MUGRIDSL'"') {
			local MUGRIDSL = `YTILDE' - (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSL'
		if missing(`"`MUGRIDSU'"') {
			local MUGRIDSU = `YTILDE' + (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSU'
		local SIZE = 1-`GAMMA'/2
		local MUGRIDS = "`MUGRIDSL' `MUGRIDSU'"
		noisily di "new mugrid is `MUGRIDS'"
  
		local MUGRIDSU_plus = `MUGRIDSU' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSL_plus = `MUGRIDSL' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSU_minus = `MUGRIDSU' - `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSL_minus = `MUGRIDSL' - `CV_UNIF_BETA' * `SIGMA'
			
		clear
		set obs 2
			
		gen mugrids_plus = .
		replace mugrids_plus = `MUGRIDSL_plus' in 1
		replace mugrids_plus = `MUGRIDSU_plus' in 2
		
		gen mugrids_minus = .
		replace mugrids_minus = `MUGRIDSL_minus' in 1
		replace mugrids_minus = `MUGRIDSU_minus' in 2
		
		gen L = `LTILDE'
		gen U = `UTILDE'
		
		egen lbound = rowmax(L mugrids_minus)
		egen ubound = rowmin(U mugrids_plus)

		if lbound[1] < ubound[1] {
			**ptrn
			local lower = (lbound[1]-`MUGRIDSL')/`SIGMA'
			local upper = (ubound[1]-`MUGRIDSL')/`SIGMA'
			local cdf_val_1 = (`YTILDE'-`MUGRIDSL')/`SIGMA'

			cap gen ptrn = .
			replace ptrn = (normal(`cdf_val_1') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 1
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[1] - (1-`SIZE') in 1
		}
		
		if lbound[2] < ubound[2] {
			**ptrn
			local lower = (lbound[2]-`MUGRIDSU')/`SIGMA'
			local upper = (ubound[2]-`MUGRIDSU')/`SIGMA'
			local cdf_val_2 = (`YTILDE'-`MUGRIDSU')/`SIGMA'

			cap gen ptrn = . 
			replace ptrn = (normal(`cdf_val_2') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 2
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[2] - (1-`SIZE') in 2
		}
		
		replace tail = 2 if missing(tail)
		
		gen sign = . 
		replace sign = -1 if tail < 0
		replace sign = 0 if tail == 0
		replace sign = 1 if tail > 0
		
		egen max_sign = max(sign)
		egen min_sign = min(sign)
		gen abs_diff = abs(max_sign-min_sign)
		
		* halt condition 
		if abs_diff > `TOL' {
			noisily di "function crosses zero after `iter' expansions"
			continue, break
		}
		else {
			local SCALE = 2 * `K'
			continue 
		}	
	}
}
macro drop _MUGRIDS _lower* _upper* _cdf_val* _tail* _PREVIOUS_LINE _MUGRIDSL ///
_MUGRIDSU _MUGRIDSM _num

clear
qui{
	forvalues iter=1/1000 {
		if missing(`"`MUGRIDS'"') {
			local MUGRIDS "0 0 0"
		}
		local PREVIOUS_LINE `MUGRIDS'
		*noisily di "previous line is `PREVIOUS_LINE'"
		local SIGMA = sqrt(`SIGMAYTILDE')
		local SCALE `K' 
		if missing(`"`MUGRIDSL'"') {
			local MUGRIDSL = `YTILDE' - (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSL'
		if missing(`"`MUGRIDSU'"') {
			local MUGRIDSU = `YTILDE' + (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSU'
		local MUGRIDSM = (`MUGRIDSL' + `MUGRIDSU')/2
		local SIZE = 1-`GAMMA'/2
		local MUGRIDS = "`MUGRIDSL' `MUGRIDSM' `MUGRIDSU'"
		noisily di "new mugrid is `MUGRIDS'"
  
		local MUGRIDSU_plus = `MUGRIDSU' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSM_plus = `MUGRIDSM' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSL_plus = `MUGRIDSL' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSU_minus = `MUGRIDSU' - `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSM_minus = `MUGRIDSM' - `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSL_minus = `MUGRIDSL' - `CV_UNIF_BETA' * `SIGMA'
			
		clear
		set obs 3
			
		gen mugrids_plus = .
		replace mugrids_plus = `MUGRIDSL_plus' in 1
		replace mugrids_plus = `MUGRIDSM_plus' in 2
		replace mugrids_plus = `MUGRIDSU_plus' in 3
		
		gen mugrids_minus = .
		replace mugrids_minus = `MUGRIDSL_minus' in 1
		replace mugrids_minus = `MUGRIDSM_minus' in 2
		replace mugrids_minus = `MUGRIDSU_minus' in 3
		
		gen L = `LTILDE'
		gen U = `UTILDE'
		
		egen lbound = rowmax(L mugrids_minus)
		egen ubound = rowmin(U mugrids_plus)

		if lbound[1] < ubound[1] {
			**ptrn
			local lower = (lbound[1]-`MUGRIDSL')/`SIGMA'
			local upper = (ubound[1]-`MUGRIDSL')/`SIGMA'
			local cdf_val_1 = (`YTILDE'-`MUGRIDSL')/`SIGMA'

			cap gen ptrn = .
			replace ptrn = (normal(`cdf_val_1') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 1
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[1] - (1-`SIZE') in 1
		}
		
		if lbound[2] < ubound[2] {
			**ptrn
			local lower = (lbound[2]-`MUGRIDSM')/`SIGMA'
			local upper = (ubound[2]-`MUGRIDSM')/`SIGMA'
			local cdf_val_2 = (`YTILDE'-`MUGRIDSM')/`SIGMA'

			cap gen ptrn = . 
			replace ptrn = (normal(`cdf_val_2') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 2
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[2] - (1-`SIZE') in 2
		}
		
		if lbound[3] < ubound[3] {
			**ptrn
			local lower = (lbound[3]-`MUGRIDSU')/`SIGMA'
			local upper = (ubound[3]-`MUGRIDSU')/`SIGMA'
			local cdf_val_3 = (`YTILDE'-`MUGRIDSU')/`SIGMA'

			cap gen ptrn = . 
			replace ptrn = (normal(`cdf_val_3') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 3
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[3] - (1-`SIZE') in 3
		}
		
	tokenize `PREVIOUS_LINE'
	gen prev_line = .
	replace prev_line = `1' in 1
	replace prev_line = `2' in 2
	replace prev_line = `3' in 3
	
	tokenize `MUGRIDS'
	gen mugrids = .
	replace mugrids = `1' in 1 
	replace mugrids = `2' in 2
	replace mugrids = `3' in 3
	
	replace tail = 2 if missing(tail)
	gen sign = . 
	replace sign = -1 if tail < 0
	replace sign = 0 if tail == 0
	replace sign = 1 if tail > 0
	
	gen diff = abs(mugrids[3] - mugrids[1])
	
	* halt conditions 
	if diff < `TOL' {
		noisily di "stopping"
		continue, break
	}
	
	gen abs_int_2 = abs(tail[2])
	if abs_int_2 < `TOL' {
		noisily di "stopping"
		continue, break
	}
	
	gen abs_mugrids_u_l_diff = abs(mugrids[3]-mugrids[1])
	if abs_mugrids_u_l_diff < `TOL'{
		noisily di "stopping"
		continue, break
	} 
	
	if sign[1] == sign[2] {
		local MUGRIDSL `MUGRIDSM'
		*noisily di `MUGRIDSL'
		noisily di "L updated, continuing.."
		continue 
	}
	
	if sign[3] == sign[2] {
		local MUGRIDSU `MUGRIDSM'
		*noisily di `MUGRIDSU'
		noisily di "U updated, continuing..."
		continue 
	}
	
	}
}
macro drop _MUGRIDS _lower* _upper* _cdf_val* _tail* _PREVIOUS_LINE _MUGRIDSL ///
_MUGRIDSU _MUGRIDSM _num
global hyb_eq_ci_upper = mugrids[2]

/*------------------------------------------------------------------------------
Step 7: Hybrid Equal-Tailed CI Lower
------------------------------------------------------------------------------*/

clear
local SCALE `K'
qui{
	forvalues iter=1/1000 {
		if missing(`"`MUGRIDS'"') {
			local MUGRIDS "0 0"
		}
		local PREVIOUS_LINE `MUGRIDS'
		*noisily di "previous line is `PREVIOUS_LINE'"
		local SIGMA = sqrt(`SIGMAYTILDE')
		local SCALE `K' 
		if missing(`"`MUGRIDSL'"') {
			local MUGRIDSL = `YTILDE' - (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSL'
		if missing(`"`MUGRIDSU'"') {
			local MUGRIDSU = `YTILDE' + (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSU'
		local SIZE = `GAMMA'/2
		local MUGRIDS = "`MUGRIDSL' `MUGRIDSU'"
		noisily di "new mugrid is `MUGRIDS'"
  
		local MUGRIDSU_plus = `MUGRIDSU' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSL_plus = `MUGRIDSL' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSU_minus = `MUGRIDSU' - `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSL_minus = `MUGRIDSL' - `CV_UNIF_BETA' * `SIGMA'
			
		clear
		set obs 2
			
		gen mugrids_plus = .
		replace mugrids_plus = `MUGRIDSL_plus' in 1
		replace mugrids_plus = `MUGRIDSU_plus' in 2
		
		gen mugrids_minus = .
		replace mugrids_minus = `MUGRIDSL_minus' in 1
		replace mugrids_minus = `MUGRIDSU_minus' in 2
		
		gen L = `LTILDE'
		gen U = `UTILDE'
		
		egen lbound = rowmax(L mugrids_minus)
		egen ubound = rowmin(U mugrids_plus)

		if lbound[1] < ubound[1] {
			**ptrn
			local lower = (lbound[1]-`MUGRIDSL')/`SIGMA'
			local upper = (ubound[1]-`MUGRIDSL')/`SIGMA'
			local cdf_val_1 = (`YTILDE'-`MUGRIDSL')/`SIGMA'

			cap gen ptrn = .
			replace ptrn = (normal(`cdf_val_1') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 1
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[1] - (1-`SIZE') in 1
		}
		
		if lbound[2] < ubound[2] {
			**ptrn
			local lower = (lbound[2]-`MUGRIDSU')/`SIGMA'
			local upper = (ubound[2]-`MUGRIDSU')/`SIGMA'
			local cdf_val_2 = (`YTILDE'-`MUGRIDSU')/`SIGMA'

			cap gen ptrn = . 
			replace ptrn = (normal(`cdf_val_2') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 2
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[2] - (1-`SIZE') in 2
		}
		
		replace tail = 2 if missing(tail)
		
		gen sign = . 
		replace sign = -1 if tail < 0
		replace sign = 0 if tail == 0
		replace sign = 1 if tail > 0
		
		egen max_sign = max(sign)
		egen min_sign = min(sign)
		gen abs_diff = abs(max_sign-min_sign)
		
		* halt condition 
		if abs_diff > `TOL' {
			noisily di "function crosses zero after `iter' expansions"
			continue, break
		}
		else {
			local SCALE = 2 * `K'
			continue 
		}	
	}
}
macro drop _MUGRIDS _lower* _upper* _cdf_val* _tail* _PREVIOUS_LINE _MUGRIDSL ///
_MUGRIDSU _MUGRIDSM _num

clear
qui{
	forvalues iter=1/1000 {
		if missing(`"`MUGRIDS'"') {
			local MUGRIDS "0 0 0"
		}
		local PREVIOUS_LINE `MUGRIDS'
		*noisily di "previous line is `PREVIOUS_LINE'"
		local SIGMA = sqrt(`SIGMAYTILDE')
		local SCALE `K' 
		if missing(`"`MUGRIDSL'"') {
			local MUGRIDSL = `YTILDE' - (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSL'
		if missing(`"`MUGRIDSU'"') {
			local MUGRIDSU = `YTILDE' + (`SCALE' * `SIGMA')
		}
		*noisily di `MUGRIDSU'
		local MUGRIDSM = (`MUGRIDSL' + `MUGRIDSU')/2
		local SIZE = `GAMMA'/2
		local MUGRIDS = "`MUGRIDSL' `MUGRIDSM' `MUGRIDSU'"
		noisily di "new mugrid is `MUGRIDS'"
  
		local MUGRIDSU_plus = `MUGRIDSU' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSM_plus = `MUGRIDSM' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSL_plus = `MUGRIDSL' + `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSU_minus = `MUGRIDSU' - `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSM_minus = `MUGRIDSM' - `CV_UNIF_BETA' * `SIGMA'
		local MUGRIDSL_minus = `MUGRIDSL' - `CV_UNIF_BETA' * `SIGMA'
			
		clear
		set obs 3
			
		gen mugrids_plus = .
		replace mugrids_plus = `MUGRIDSL_plus' in 1
		replace mugrids_plus = `MUGRIDSM_plus' in 2
		replace mugrids_plus = `MUGRIDSU_plus' in 3
		
		gen mugrids_minus = .
		replace mugrids_minus = `MUGRIDSL_minus' in 1
		replace mugrids_minus = `MUGRIDSM_minus' in 2
		replace mugrids_minus = `MUGRIDSU_minus' in 3
		
		gen L = `LTILDE'
		gen U = `UTILDE'
		
		egen lbound = rowmax(L mugrids_minus)
		egen ubound = rowmin(U mugrids_plus)

		if lbound[1] < ubound[1] {
			**ptrn
			local lower = (lbound[1]-`MUGRIDSL')/`SIGMA'
			local upper = (ubound[1]-`MUGRIDSL')/`SIGMA'
			local cdf_val_1 = (`YTILDE'-`MUGRIDSL')/`SIGMA'

			cap gen ptrn = .
			replace ptrn = (normal(`cdf_val_1') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 1
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[1] - (1-`SIZE') in 1
		}
		
		if lbound[2] < ubound[2] {
			**ptrn
			local lower = (lbound[2]-`MUGRIDSM')/`SIGMA'
			local upper = (ubound[2]-`MUGRIDSM')/`SIGMA'
			local cdf_val_2 = (`YTILDE'-`MUGRIDSM')/`SIGMA'

			cap gen ptrn = . 
			replace ptrn = (normal(`cdf_val_2') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 2
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[2] - (1-`SIZE') in 2
		}
		
		if lbound[3] < ubound[3] {
			**ptrn
			local lower = (lbound[3]-`MUGRIDSU')/`SIGMA'
			local upper = (ubound[3]-`MUGRIDSU')/`SIGMA'
			local cdf_val_3 = (`YTILDE'-`MUGRIDSU')/`SIGMA'

			cap gen ptrn = . 
			replace ptrn = (normal(`cdf_val_3') - normal(`lower')) / (normal(`upper') - normal(`lower')) in 3
			*replace ptrn = 0 if missing(ptrn)
				 
			cap gen tail = . 
			replace tail = ptrn[3] - (1-`SIZE') in 3
		}
		
	tokenize `PREVIOUS_LINE'
	gen prev_line = .
	replace prev_line = `1' in 1
	replace prev_line = `2' in 2
	replace prev_line = `3' in 3
	
	tokenize `MUGRIDS'
	gen mugrids = .
	replace mugrids = `1' in 1 
	replace mugrids = `2' in 2
	replace mugrids = `3' in 3
	
	replace tail = 2 if missing(tail)
	gen sign = . 
	replace sign = -1 if tail < 0
	replace sign = 0 if tail == 0
	replace sign = 1 if tail > 0
	
	gen diff = abs(mugrids[3] - mugrids[1])
	
	* halt conditions 
	if diff < `TOL' {
		noisily di "stopping"
		continue, break
	}
	
	gen abs_int_2 = abs(tail[2])
	if abs_int_2 < `TOL' {
		noisily di "stopping"
		continue, break
	}
	
	gen abs_mugrids_u_l_diff = abs(mugrids[3]-mugrids[1])
	if abs_mugrids_u_l_diff < `TOL'{
		noisily di "stopping"
		continue, break
	} 
	
	if sign[1] == sign[2] {
		local MUGRIDSL `MUGRIDSM'
		*noisily di `MUGRIDSL'
		noisily di "L updated, continuing.."
		continue 
	}
	
	if sign[3] == sign[2] {
		local MUGRIDSU `MUGRIDSM'
		*noisily di `MUGRIDSU'
		noisily di "U updated, continuing..."
		continue 
	}
	
	}
}
macro drop _MUGRIDS _lower* _upper* _cdf_val* _tail* _PREVIOUS_LINE _MUGRIDSL ///
_MUGRIDSU _MUGRIDSM _num
global hyb_eq_ci_lower = mugrids[2]

/*------------------------------------------------------------------------------
Step 8: Projection CI
------------------------------------------------------------------------------*/

global proj_ci_upper = `YTILDE' + `CV_UNIF' * `SIGMA'
global proj_ci_lower = `YTILDE' - `CV_UNIF' * `SIGMA'

/*------------------------------------------------------------------------------
Step 9: Print Output
------------------------------------------------------------------------------*/

noisily macro li median_unbiased_estimate equi_ci_upper equi_ci_lower hyb_med_est ///
hyb_eq_ci_upper hyb_eq_ci_lower proj_ci_upper proj_ci_lower

/*------------------------------------------------------------------------------
									END
------------------------------------------------------------------------------*/